@@ -17,15 +17,9 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#include <errno.h>
-#include <stdlib.h>
-#include <string.h>
-
 #define FP_COMPONENT "vfs101"
 
-#include <fp_internal.h>
-
-#include "driver_ids.h"
+#include "drivers_api.h"
 
 /* Input-Output usb endpoint */
 #define EP_IN(n)	(n | LIBUSB_ENDPOINT_IN)
@@ -190,7 +184,7 @@ static int result_code(struct fp_img_dev *dev, int result)
 		return result;
 
 	/* Return result code */
-	if (dev->action == IMG_ACTION_ENROLL)
+	if (fpi_imgdev_get_action(dev) == IMG_ACTION_ENROLL)
 		return result_codes[0][result];
 	else
 		return result_codes[1][result];
@@ -206,8 +200,8 @@ static int result_code(struct fp_img_dev *dev, int result)
 static void async_send_cb(struct libusb_transfer *transfer)
 {
 	struct fpi_ssm *ssm = transfer->user_data;
-	struct fp_img_dev *dev = ssm->priv;
-	struct vfs101_dev *vdev = dev->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
+	struct vfs101_dev *vdev = fpi_imgdev_get_user_data(dev);
 
 	/* Cleanup transfer */
 	vdev->transfer = NULL;
@@ -250,8 +244,8 @@ out:
 /* Submit asynchronous send */
 static void async_send(struct fpi_ssm *ssm)
 {
-	struct fp_img_dev *dev = ssm->priv;
-	struct vfs101_dev *vdev = dev->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
+	struct vfs101_dev *vdev = fpi_imgdev_get_user_data(dev);
 	int r;
 
 	/* Allocation of transfer */
@@ -271,7 +265,7 @@ static void async_send(struct fpi_ssm *ssm)
 	vdev->buffer[1] = byte(1, vdev->seqnum);
 
 	/* Prepare bulk transfer */
-	libusb_fill_bulk_transfer(vdev->transfer, dev->udev, EP_OUT(1), vdev->buffer, vdev->length, async_send_cb, ssm, BULK_TIMEOUT);
+	libusb_fill_bulk_transfer(vdev->transfer, fpi_imgdev_get_usb_dev(dev), EP_OUT(1), vdev->buffer, vdev->length, async_send_cb, ssm, BULK_TIMEOUT);
 
 	/* Submit transfer */
 	r = libusb_submit_transfer(vdev->transfer);
@@ -290,8 +284,8 @@ static void async_send(struct fpi_ssm *ssm)
 static void async_recv_cb(struct libusb_transfer *transfer)
 {
 	struct fpi_ssm *ssm = transfer->user_data;
-	struct fp_img_dev *dev = ssm->priv;
-	struct vfs101_dev *vdev = dev->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
+	struct vfs101_dev *vdev = fpi_imgdev_get_user_data(dev);
 
 	/* Cleanup transfer */
 	vdev->transfer = NULL;
@@ -337,8 +331,8 @@ out:
 /* Submit asynchronous recv */
 static void async_recv(struct fpi_ssm *ssm)
 {
-	struct fp_img_dev *dev = ssm->priv;
-	struct vfs101_dev *vdev = dev->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
+	struct vfs101_dev *vdev = fpi_imgdev_get_user_data(dev);
 	int r;
 
 	/* Allocation of transfer */
@@ -353,7 +347,7 @@ static void async_recv(struct fpi_ssm *ssm)
 	}
 
 	/* Prepare bulk transfer */
-	libusb_fill_bulk_transfer(vdev->transfer, dev->udev, EP_IN(1), vdev->buffer, 0x0f, async_recv_cb, ssm, BULK_TIMEOUT);
+	libusb_fill_bulk_transfer(vdev->transfer, fpi_imgdev_get_usb_dev(dev), EP_IN(1), vdev->buffer, 0x0f, async_recv_cb, ssm, BULK_TIMEOUT);
 
 	/* Submit transfer */
 	r = libusb_submit_transfer(vdev->transfer);
@@ -374,8 +368,8 @@ static void async_load(struct fpi_ssm *ssm);
 static void async_load_cb(struct libusb_transfer *transfer)
 {
 	struct fpi_ssm *ssm = transfer->user_data;
-	struct fp_img_dev *dev = ssm->priv;
-	struct vfs101_dev *vdev = dev->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
+	struct vfs101_dev *vdev = fpi_imgdev_get_user_data(dev);
 
 	/* Cleanup transfer */
 	vdev->transfer = NULL;
@@ -438,8 +432,8 @@ out:
 /* Submit asynchronous load */
 static void async_load(struct fpi_ssm *ssm)
 {
-	struct fp_img_dev *dev = ssm->priv;
-	struct vfs101_dev *vdev = dev->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
+	struct vfs101_dev *vdev = fpi_imgdev_get_user_data(dev);
 	unsigned char *buffer;
 	int r;
 
@@ -458,7 +452,7 @@ static void async_load(struct fpi_ssm *ssm)
 	buffer = vdev->buffer + vdev->length;
 
 	/* Prepare bulk transfer */
-	libusb_fill_bulk_transfer(vdev->transfer, dev->udev, EP_IN(2), buffer, VFS_BLOCK_SIZE, async_load_cb, ssm, BULK_TIMEOUT);
+	libusb_fill_bulk_transfer(vdev->transfer, fpi_imgdev_get_usb_dev(dev), EP_IN(2), buffer, VFS_BLOCK_SIZE, async_load_cb, ssm, BULK_TIMEOUT);
 
 	/* Submit transfer */
 	r = libusb_submit_transfer(vdev->transfer);
@@ -477,8 +471,8 @@ static void async_load(struct fpi_ssm *ssm)
 static void async_sleep_cb(void *data)
 {
 	struct fpi_ssm *ssm = data;
-	struct fp_img_dev *dev = ssm->priv;
-	struct vfs101_dev *vdev = dev->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
+	struct vfs101_dev *vdev = fpi_imgdev_get_user_data(dev);
 
 	/* Cleanup timeout */
 	vdev->timeout = NULL;
@@ -489,8 +483,8 @@ static void async_sleep_cb(void *data)
 /* Submit asynchronous sleep */
 static void async_sleep(unsigned int msec, struct fpi_ssm *ssm)
 {
-	struct fp_img_dev *dev = ssm->priv;
-	struct vfs101_dev *vdev = dev->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
+	struct vfs101_dev *vdev = fpi_imgdev_get_user_data(dev);
 
 	/* Add timeout */
 	vdev->timeout = fpi_timeout_add(msec, async_sleep_cb, ssm);
@@ -515,7 +509,7 @@ enum
 /* Exec swap sequential state machine */
 static void m_swap_state(struct fpi_ssm *ssm)
 {
-	switch (ssm->cur_state)
+	switch (fpi_ssm_get_cur_state(ssm))
 	{
 	case M_SWAP_SEND:
 		/* Send data */
@@ -532,8 +526,8 @@ static void m_swap_state(struct fpi_ssm *ssm)
 /* Start swap sequential state machine */
 static void m_swap(struct fpi_ssm *ssm, unsigned char *data, size_t length)
 {
-	struct fp_img_dev *dev = ssm->priv;
-	struct vfs101_dev *vdev = dev->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
+	struct vfs101_dev *vdev = fpi_imgdev_get_user_data(dev);
 	struct fpi_ssm *subsm;
 
 	/* Prepare data for sending */
@@ -542,8 +536,8 @@ static void m_swap(struct fpi_ssm *ssm, unsigned char *data, size_t length)
 	vdev->length = length;
 
 	/* Start swap ssm */
-	subsm = fpi_ssm_new(dev->dev, m_swap_state, M_SWAP_NUM_STATES);
-	subsm->priv = dev;
+	subsm = fpi_ssm_new(fpi_imgdev_get_dev(dev), m_swap_state, M_SWAP_NUM_STATES);
+	fpi_ssm_set_user_data(subsm, dev);
 	fpi_ssm_start_subsm(ssm, subsm);
 }
 
@@ -589,7 +583,7 @@ static void vfs_abort_print(struct fpi_ssm *ssm)
 {
 	unsigned char data[0x06] = { 0x00, 0x00, 0x00, 0x00, 0x0E, 0x00 };
 
-	fp_dbg("");
+	G_DEBUG_HERE();
 
 	/* Run swap sequential state machine */
 	m_swap (ssm, data, 0x06);
@@ -622,7 +616,7 @@ static void vfs_get_finger_state(struct fpi_ssm *ssm)
 {
 	unsigned char data[0x06] = { 0x00, 0x00, 0x00, 0x00, 0x16, 0x00 };
 
-	fp_dbg("");
+	G_DEBUG_HERE();
 
 	/* Run swap sequential state machine */
 	m_swap (ssm, data, 0x06);
@@ -631,10 +625,10 @@ static void vfs_get_finger_state(struct fpi_ssm *ssm)
 /* Load raw image from reader */
 static void vfs_img_load(struct fpi_ssm *ssm)
 {
-	struct fp_img_dev *dev = ssm->priv;
-	struct vfs101_dev *vdev = dev->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
+	struct vfs101_dev *vdev = fpi_imgdev_get_user_data(dev);
 
-	fp_dbg("");
+	G_DEBUG_HERE();
 
 	/* Reset buffer length */
 	vdev->length = 0;
@@ -650,10 +644,10 @@ static void vfs_img_load(struct fpi_ssm *ssm)
 /* Check if action is completed */
 static int action_completed(struct fp_img_dev *dev)
 {
-	struct vfs101_dev *vdev = dev->priv;
+	struct vfs101_dev *vdev = fpi_imgdev_get_user_data(dev);
 
-	if ((dev->action == IMG_ACTION_ENROLL) &&
-		(vdev->enroll_stage < dev->dev->nr_enroll_stages))
+	if ((fpi_imgdev_get_action(dev) == IMG_ACTION_ENROLL) &&
+		(vdev->enroll_stage < fpi_dev_get_nr_enroll_stages(fpi_imgdev_get_dev(dev))))
 		/* Enroll not completed, return false */
 		return FALSE;
 
@@ -762,8 +756,8 @@ static void img_copy(struct vfs101_dev *vdev, struct fp_img *img)
 /* Extract fingerpint image from raw data */
 static void img_extract(struct fpi_ssm *ssm)
 {
-	struct fp_img_dev *dev = ssm->priv;
-	struct vfs101_dev *vdev = dev->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
+	struct vfs101_dev *vdev = fpi_imgdev_get_user_data(dev);
 	struct fp_img *img;
 
 	/* Screen image to remove noise and find top and bottom line */
@@ -793,28 +787,28 @@ static void img_extract(struct fpi_ssm *ssm)
 	fpi_imgdev_image_captured(dev, img);
 
 	/* Check captured result */
-	if (dev->action_result >= 0 &&
-		dev->action_result != FP_ENROLL_RETRY &&
-		dev->action_result != FP_VERIFY_RETRY)
+	if (fpi_imgdev_get_action_result(dev) >= 0 &&
+		fpi_imgdev_get_action_result(dev) != FP_ENROLL_RETRY &&
+		fpi_imgdev_get_action_result(dev) != FP_VERIFY_RETRY)
 	{
 		/* Image captured, increase enroll stage */
 		vdev->enroll_stage++;
 
 		/* Check if action is completed */
 		if (!action_completed(dev))
-			dev->action_result = FP_ENROLL_PASS;
+			fpi_imgdev_set_action_result(dev, FP_ENROLL_PASS);
 	}
 	else
 	{
 		/* Image capture failed */
-		if (dev->action == IMG_ACTION_ENROLL)
+		if (fpi_imgdev_get_action(dev) == IMG_ACTION_ENROLL)
 			/* Return retry */
-			dev->action_result = result_code(dev, RESULT_RETRY);
+			fpi_imgdev_set_action_result(dev, result_code(dev, RESULT_RETRY));
 		else
 		{
 			/* Return no match */
 			vdev->enroll_stage++;
-			dev->action_result = FP_VERIFY_NO_MATCH;
+			fpi_imgdev_set_action_result(dev, FP_VERIFY_NO_MATCH);
 		}
 	}
 
@@ -920,8 +914,8 @@ enum
 /* Exec loop sequential state machine */
 static void m_loop_state(struct fpi_ssm *ssm)
 {
-	struct fp_img_dev *dev = ssm->priv;
-	struct vfs101_dev *vdev = dev->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
+	struct vfs101_dev *vdev = fpi_imgdev_get_user_data(dev);
 
 	/* Check action state */
 	if (!vdev->active)
@@ -931,7 +925,7 @@ static void m_loop_state(struct fpi_ssm *ssm)
 		return;
 	}
 
-	switch (ssm->cur_state)
+	switch (fpi_ssm_get_cur_state(ssm))
 	{
 	case M_LOOP_0_GET_PRINT:
 		/* Send get print command to the reader */
@@ -1184,8 +1178,8 @@ enum
 /* Exec init sequential state machine */
 static void m_init_state(struct fpi_ssm *ssm)
 {
-	struct fp_img_dev *dev = ssm->priv;
-	struct vfs101_dev *vdev = dev->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
+	struct vfs101_dev *vdev = fpi_imgdev_get_user_data(dev);
 
 	/* Check action state */
 	if (!vdev->active)
@@ -1195,7 +1189,7 @@ static void m_init_state(struct fpi_ssm *ssm)
 		return;
 	}
 
-	switch (ssm->cur_state)
+	switch (fpi_ssm_get_cur_state(ssm))
 	{
 	case M_INIT_0_RECV_DIRTY:
 		/* Recv eventualy dirty data */
@@ -1425,18 +1419,18 @@ static void m_init_state(struct fpi_ssm *ssm)
 /* Complete init sequential state machine */
 static void m_init_complete(struct fpi_ssm *ssm)
 {
-	struct fp_img_dev *dev = ssm->priv;
-	struct vfs101_dev *vdev = dev->priv;
+	struct fp_img_dev *dev = fpi_ssm_get_user_data(ssm);
+	struct vfs101_dev *vdev = fpi_imgdev_get_user_data(dev);
 	struct fpi_ssm *ssm_loop;
 
-	if (!ssm->error && vdev->active)
+	if (!fpi_ssm_get_error(ssm) && vdev->active)
 	{
 		/* Notify activate complete */
 		fpi_imgdev_activate_complete(dev, 0);
 
 		/* Start loop ssm */
-		ssm_loop = fpi_ssm_new(dev->dev, m_loop_state, M_LOOP_NUM_STATES);
-		ssm_loop->priv = dev;
+		ssm_loop = fpi_ssm_new(fpi_imgdev_get_dev(dev), m_loop_state, M_LOOP_NUM_STATES);
+		fpi_ssm_set_user_data(ssm_loop, dev);
 		fpi_ssm_start(ssm_loop, m_loop_complete);
 	}
 
@@ -1447,7 +1441,7 @@ static void m_init_complete(struct fpi_ssm *ssm)
 /* Activate device */
 static int dev_activate(struct fp_img_dev *dev, enum fp_imgdev_state state)
 {
-	struct vfs101_dev *vdev = dev->priv;
+	struct vfs101_dev *vdev = fpi_imgdev_get_user_data(dev);
 	struct fpi_ssm *ssm;
 
 	/* Check if already active */
@@ -1470,8 +1464,8 @@ static int dev_activate(struct fp_img_dev *dev, enum fp_imgdev_state state)
 	vdev->enroll_stage = 0;
 
 	/* Start init ssm */
-	ssm = fpi_ssm_new(dev->dev, m_init_state, M_INIT_NUM_STATES);
-	ssm->priv = dev;
+	ssm = fpi_ssm_new(fpi_imgdev_get_dev(dev), m_init_state, M_INIT_NUM_STATES);
+	fpi_ssm_set_user_data(ssm, dev);
 	fpi_ssm_start(ssm, m_init_complete);
 
 	return 0;
@@ -1480,7 +1474,7 @@ static int dev_activate(struct fp_img_dev *dev, enum fp_imgdev_state state)
 /* Deactivate device */
 static void dev_deactivate(struct fp_img_dev *dev)
 {
-	struct vfs101_dev *vdev = dev->priv;
+	struct vfs101_dev *vdev = fpi_imgdev_get_user_data(dev);
 
 	/* Reset active state */
 	vdev->active = FALSE;
@@ -1500,7 +1494,7 @@ static int dev_open(struct fp_img_dev *dev, unsigned long driver_data)
 	int r;
 
 	/* Claim usb interface */
-	r = libusb_claim_interface(dev->udev, 0);
+	r = libusb_claim_interface(fpi_imgdev_get_usb_dev(dev), 0);
 	if (r < 0)
 	{
 		/* Interface not claimed, return error */
@@ -1511,7 +1505,7 @@ static int dev_open(struct fp_img_dev *dev, unsigned long driver_data)
 	/* Initialize private structure */
 	vdev = g_malloc0(sizeof(struct vfs101_dev));
 	vdev->seqnum = -1;
-	dev->priv = vdev;
+	fpi_imgdev_set_user_data(dev, vdev);
 
 	/* Notify open complete */
 	fpi_imgdev_open_complete(dev, 0);
@@ -1522,11 +1516,14 @@ static int dev_open(struct fp_img_dev *dev, unsigned long driver_data)
 /* Close device */
 static void dev_close(struct fp_img_dev *dev)
 {
+	struct vfs101_dev *vdev;
+
 	/* Release private structure */
-	g_free(dev->priv);
+	vdev = fpi_imgdev_get_user_data(dev);
+	g_free(vdev);
 
 	/* Release usb interface */
-	libusb_release_interface(dev->udev, 0);
+	libusb_release_interface(fpi_imgdev_get_usb_dev(dev), 0);
 
 	/* Notify close complete */
 	fpi_imgdev_close_complete(dev);
